//     Copyright 2011 Reuben D'Netto
//
//     This file is part of SMWebUI.
//
//     SMWebUI is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     SMWebUI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with SMWebUI.  If not, see <http://www.gnu.org/licenses/>.


#ifndef MYAPPLICATION_H
#define MYAPPLICATION_H

#include <QApplication>
#include <QTcpServer>
#include <QSettings>
#include <QDir>
#include <QNetworkInterface>
#include "smplayerconnection.h"

class MyApplication : public QApplication{ Q_OBJECT
private:
	QTcpServer* server;
	SmplayerConnection* connection;

	static QString ErrorMsg(int errNo, QString name){
		return	QString("HTTP/1.0 %1 %2\r\n"
						"Connection: close\r\n"
						"Content-Type: text/html\r\n\r\n"
						"<html><head><title>%1 - %2</title></head><body><h1>%1 - %2</h1></body></html>")
				.arg(QString::number(errNo), name);
	}

	static QString GetMimeType(QString ext){
		if(ext == "html" || ext == "htm" || ext == "csv" || ext == "css" || ext == "xml") return "text/" + ext;
		else if(ext == "txt") return "text/plain";
		else if(ext == "js")  return "text/javascript";
		else if(ext == "gif" || ext == "png" || ext == "tiff") return "image/" + ext;
		else if(ext == "jpg" || ext == "jpeg") return "image/jpeg";
		else if(ext == "ico") return "image/vnd.microsoft.icon";
		else return "";
	}

	static QString getAddresses(){
		QString result = "";
		QHostAddress adr;

		foreach(adr, QNetworkInterface::allAddresses())
			result += adr.toString() + ", ";

		result.remove(result.length() - 2, 2);
		return result;
	}

	void LoadSmplayerSettings(){
		QSettings *settings;

		#ifndef Q_OS_WIN
		QString iniPath = QDir::homePath() + "/.config/smplayer";
		#else
		QString iniPath = QDir::homePath() + "/.smplayer";
		#endif

		QString filename = iniPath + "/smplayer.ini";
		settings = new QSettings(filename, QSettings::IniFormat);

		qDebug("MyApplication::LoadSmplayerSettings: %s", filename.toUtf8().data());
		qDebug("MyApplication::LoadSmplayerSettings: use_autoport = %s", settings->value("instances/use_autoport").toBool() ? "true" : "false");
		qDebug("MyApplication::LoadSmplayerSettings: temp/autoport = %i", settings->value("instances/temp/autoport").toInt());
		qDebug("MyApplication::LoadSmplayerSettings: connection_port = %i", settings->value("instances/connection_port").toInt());


		if(settings->value("instances/use_autoport", false).toBool())
			SmplayerPort = settings->value("instances/temp/autoport", SmplayerPort).toInt();
		else
			SmplayerPort = settings->value("instances/connection_port", SmplayerPort).toInt();

		settings->deleteLater();
	}

public:
	int HttpPort;
	int SmplayerPort;
	QString SmplayerHost;

	MyApplication(int argc, char *argv[]);

	~MyApplication(){
		delete server;
		delete connection;
	}

	int exec();


signals:

private slots:
	void NewConnection();

};

#endif // MYAPPLICATION_H
