//     Copyright 2011 Reuben D'Netto
//
//     This file is part of SMWebUI.
//
//     SMWebUI is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     SMWebUI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with SMWebUI.  If not, see <http://www.gnu.org/licenses/>.


#ifndef SMPLAYERCONNECTION_H
#define SMPLAYERCONNECTION_H

#define SMPLAYERCONNECTION_CHECKVERSION
//if defined, then the version is compared directly
//otherwise the help command is used to check for command support

#include <QObject>
#include <QTcpSocket>
#include <QApplication>
#include "smplayer_structs.h"

class SmplayerConnection : public QObject{
    Q_OBJECT

private:
	bool _partialSupport;
	QApplication* app;

public:
	QTcpSocket* socket;
	bool PartialSupportOnly(){return _partialSupport;} //true if not all commands are supported

	SmplayerConnection(QApplication *parent = 0);
	~SmplayerConnection(){
		delete socket;
	}

	bool initialize(QString host, int port);
	QString sendQuery(QString);

	//smplayer functions
	void play(){	sendQuery("f play");}
	void pause(){	sendQuery("f pause");}
	void stop(){	sendQuery("f stop");}
	void previous(){sendQuery("f play_prev");}
	void next(){	sendQuery("f play_next");}
	void quit(){	sendQuery("f quit");}

	QList<PlaylistItem> playlist();
	ClipInfo clipInfo();

	void openFile(QString file){
		//do not use the 'open' command as it does not update the GUI
		sendQuery("open_files_start");
		sendQuery("open_files " + file); //this line can be repeated multiple times for multiple files
		sendQuery("open_files_end");
	}
	void addFile(QString file){
		sendQuery("add_files_start");
		sendQuery("add_files " + file); //this line can be repeated multiple times for multiple files
		sendQuery("add_files_end");
	}

	void playItem(int i){
		sendQuery("play item " + QString::number(i));
	}
	void removeItem(int i){
		sendQuery("remove item " + (i == -1 ? "*" : QString::number(i)));
	}
	void moveItem(int i, int shift){
		sendQuery(QString("move item %1 %2").arg(QString::number(i), QString::number(shift)));
	}

	void seek(int pos){
		sendQuery("seek " + QString::number(pos));
	}

	QString status(){
		QString result = sendQuery("view status");
		return (result == "" ? "Error" : result);
	}

	QString getValue(QString name){
		return sendQuery("get " + name);
	}

	int getVolume(){
		return sendQuery("get volume").toInt();
	}
	void setVolume(int value){
		sendQuery("set volume " + QString::number(value));
	}

signals:

private slots:
	void OnError(QAbstractSocket::SocketError){
		qWarning("SmplayerConnection::OnError: %s", socket->errorString().toUtf8().data());
	}
	void OnStateChanged(QAbstractSocket::SocketState state){
		switch(state){
		case QAbstractSocket::UnconnectedState:
			qDebug("SmplayerConnection: state changed: %s", "Disconnected.");
			app->exit(3);
			break;
		case QAbstractSocket::HostLookupState:
			qDebug("SmplayerConnection: state changed: %s", "Looking up hostname...");
			break;
		case QAbstractSocket::ConnectingState:
			qDebug("SmplayerConnection: state changed: %s", "Establishing connection...");
			break;
		case QAbstractSocket::ConnectedState:
			qDebug("SmplayerConnection: state changed: %s", "Connection established.");
			break;
		case QAbstractSocket::BoundState:
			qDebug("SmplayerConnection: state changed: %s", "Bound to an address and port.");
			break;
		case QAbstractSocket::ClosingState:
			qDebug("SmplayerConnection: state changed: %s", "Closing...");
			break;
		default:
			qDebug("SmplayerConnection: state changed: Unknown state %i.", state);
			break;
		} //end switch
	}
	void CheckVersion();
};

#endif // SMPLAYERCONNECTION_H
